// import 'flutter_installer_platform_interface.dart';

import 'dart:io';

import 'package:flutter/services.dart';
import 'package:path_provider/path_provider.dart';

class FlutterInstallerException implements Exception {
  final String message;
  const FlutterInstallerException(this.message);

  @override
  String toString() => message;
}

class FlutterInstaller {
  final String mainDirectory;

  const FlutterInstaller({this.mainDirectory = 'flutter_installer'});

  Future<Directory> _getDirectory() async {
    final Directory directory = await getApplicationDocumentsDirectory();
    final Directory mainDir = Directory('${directory.path}/$mainDirectory');
    if (!(await mainDir.exists())) {
      await mainDir.create();
    }
    return mainDir;
  }

  Future<File> _getFile(String name) async {
    final Directory mainDir = await _getDirectory();
    return File('${mainDir.path}/$name');
  }

  Future<String> install({
    required List<int> fileBytes,
    required String name,
    bool replace = false,
  }) async {
    try {
      File file = await _getFile(name);

      if (await file.exists()) {
        if (replace) {
          file = await file.writeAsBytes(fileBytes);
        } else {
          throw const FlutterInstallerException("File already exists.");
        }
      } else {
        file = await file.writeAsBytes(fileBytes);
      }

      return file.path;
    } catch (e) {
      throw FlutterInstallerException("Failed to save file: $e");
    }
  }

  Future<String> installFromAssets({
    required String assetPath,
    required String name,
    bool replace = false,
  }) async {
    try {
      ByteData data = await rootBundle.load(assetPath);
      List<int> bytes =
          data.buffer.asUint8List(data.offsetInBytes, data.lengthInBytes);
      return await install(fileBytes: bytes, name: name, replace: replace);
    } catch (e) {
      throw FlutterInstallerException("Failed to install file from assets: $e");
    }
  }

  Future<bool> deleteFile(String name) async {
    try {
      File file = await _getFile(name);
      await file.delete();
      return true;
    } catch (e) {
      throw FlutterInstallerException("Failed to delete file: $e");
    }
  }

  Future<void> deleteAllFiles() async {
    try {
      Directory directory = await _getDirectory();
      directory.deleteSync(recursive: true);
    } catch (e) {
      throw FlutterInstallerException("Failed to delete all files: $e");
    }
  }

  Future<bool> isFileExists(String name) async {
    File file = await _getFile(name);
    return file.exists();
  }

  Future<List<int>> readFile(String name) async {
    try {
      File file = await _getFile(name);
      return file.readAsBytes();
    } catch (e) {
      throw FlutterInstallerException("Failed to read file: $e");
    }
  }

  Future<String> copyFile({
    required String existingFileName,
    required String newFileName,
  }) async {
    try {
      File existingFile = await _getFile(existingFileName);
      File newFile = await _getFile(newFileName);

      if (await existingFile.exists()) {
        await existingFile.copy(newFile.path);
        return newFile.path;
      } else {
        throw const FlutterInstallerException("Source file does not exist.");
      }
    } catch (e) {
      throw FlutterInstallerException("Failed to copy file: $e");
    }
  }

  Future<String> renameFile({
    required String existingFileName,
    required String newFileName,
  }) async {
    try {
      File existingFile = await _getFile(existingFileName);
      File newFile = await _getFile(newFileName);

      if (await existingFile.exists()) {
        await existingFile.rename(newFile.path);
        return newFile.path;
      } else {
        throw const FlutterInstallerException("Source file does not exist.");
      }
    } catch (e) {
      throw FlutterInstallerException("Failed to rename file: $e");
    }
  }
}


// class FlutterInstaller {
//   Future<String?> getPlatformVersion() {
//     return FlutterInstallerPlatform.instance.getPlatformVersion();
//   }
// }
